@@ -1,41 +1,84 @@
-# application.py: Application class to store properties of the application
-#                 being created
-# $Id: application.py,v 1.64 2007/08/07 12:13:44 agriggio Exp $
-# 
-# Copyright (c) 2002-2007 Alberto Griggio <agriggio@users.sourceforge.net>
-# License: MIT (see license.txt)
-# THIS PROGRAM COMES WITH NO WARRANTY
+"""
+Application class to store properties of the application being created
+ 
+@copyright: 2002-2007 Alberto Griggio <agriggio@users.sourceforge.net>
+@copyright: 2012 Carsten Grohmann <mail@carstengrohmann.de>
+@license: MIT (see license.txt) - THIS PROGRAM COMES WITH NO WARRANTY
+"""
 
+import codecs
+import locale
+import os
+import re
 import wx
+
 from widget_properties import *
-import common, math, misc, os, config
-import traceback, re
+import common
+import config
+import errors
+import math
+import misc
 
 class FileDirDialog:
     """\
     Custom class which displays a FileDialog or a DirDialog, according to the
-    value of the codegen_opt of its parent (instance of Application)
+    value of the L{Application.codegen_opt} of its parent (instance of
+    L{Application}).
+    
+    @ivar default_extension: The default extension will be added to all
+                             file names without extension.
+    @type default_extension: String
+    
+    @ivar file_message: Message to show on the file dialog
+    @type file_message: String
+    
+    @ivar dir_message: Message to show on the directory dialog
+    @type dir_message: String
+    
+    @ivar file_style: Style for the file dialog
+    @ivar dir_style:  Style for the directory dialog
+    @ivar value:      Value returned by file or directory dialog on success
+    @ivar parent:     Parent instance of L{Application}
+    @ivar prev_dir:   Previous directory
     """
-    def __init__(self, owner, parent, wildcard=_("All Files|*"),
-                 file_message=_("Choose a file"), dir_message=None, style=0):
-        self.owner = owner
-        self.prev_dir = config.preferences.codegen_path
+    def __init__(self, parent, wildcard=_("All Files|*"),
+                 file_message=_("Choose a file"), dir_message=None, file_style=0):
+        self.prev_dir = config.preferences.codegen_path or ""
         self.wildcard = wildcard
         self.file_message = file_message
         self.dir_message = dir_message
-        self.file_style = style
+        self.file_style = file_style
         self.dir_style = wx.DD_DEFAULT_STYLE|wx.DD_NEW_DIR_BUTTON
         self.parent = parent
         self.value = None
+        self.default_extension = None
 
     def ShowModal(self):
-        if self.owner.codegen_opt == 0:
+        """\
+        Show a FileDialog or a DirDialog as a modal dialog. The selected
+        file or directory is stored in L{value} on success.
+        
+        @return: C{ID_OK} or C{ID_CANCEL}
+        @see: L{get_value()}
+        """
+        if self.parent.codegen_opt == 0:
             self.value = misc.FileSelector(
-                self.file_message, self.prev_dir or "",
-                wildcard=self.wildcard, flags=self.file_style)
+                self.file_message,
+                self.prev_dir,
+                wildcard=self.wildcard,
+                flags=self.file_style
+                )
+            # check for file extension and add default extension if missing
+            if self.value and self.default_extension:
+                ext = os.path.splitext(self.value)[1].lower()
+                if not ext:
+                    self.value = "%s%s" % (self.value, self.default_extension)
         else:
             self.value = misc.DirSelector(
-                self.dir_message, self.prev_dir or "", style=self.dir_style)
+                self.dir_message,
+                self.prev_dir,
+                style=self.dir_style
+                )
         if self.value:
             self.prev_dir = self.value
             if not os.path.isdir(self.prev_dir):
@@ -44,18 +87,36 @@ class FileDirDialog:
         return wx.ID_CANCEL
 
     def get_value(self):
+        """\
+        Return the dialog value returned during the last L{ShowModal()} call.
+        
+        @see: L{value}
+        """
         return self.value
 
-    def set_wildcard(self, wildcard):
-        self.wildcard = wildcard
-        
 # end of class FileDirDialog
 
 
 class Application(object):
     """\
-    properties of the application being created
+    Properties of the application being created
+    
+    @ivar __filename:  Name of the output XML file
+    @ivar __saved:     If True, there are no changes to save
+    @ivar codegen_opt: If != 0, generates a separate file for each class
+    @ivar for_version: Version string of major dot minor version number
+    @type for_version: String
+    @ivar klass:       Name of the automatically generated class derived from
+                       wxApp
+    @ivar name:        Name of the wxApp instance to generate
+    @ivar notebook:    Notebook to show different property panels
+    
+    @cvar all_supported_versions: Supported wx versions
+    @type all_supported_versions: List of strings
     """
+    
+    all_supported_versions = ['2.6', '2.8']
+
     def __init__(self, property_window):
         self.property_window = property_window
         self.notebook = wx.Notebook(self.property_window, -1)
@@ -63,26 +124,29 @@ class Application(object):
         self.notebook.SetAutoLayout(True)
         self.notebook.Hide()
         panel = wx.ScrolledWindow(
-            self.notebook, -1, style=wx.TAB_TRAVERSAL|wx.FULL_REPAINT_ON_RESIZE)
-        lang_panels = {}
-        def mkp():
-            return wx.ScrolledWindow(
-                self.notebook, -1,
-                style=wx.TAB_TRAVERSAL|wx.FULL_REPAINT_ON_RESIZE)
-        lang_panels['C++'] = mkp()
-        lang_panels['python'] = mkp()
-        self.name = "app" # name of the wxApp instance to generate
-        self.__saved = True # if True, there are no changes to save
-        self.__filename = None # name of the output xml file
+            self.notebook, 
+            wx.ID_ANY,
+            style=wx.TAB_TRAVERSAL | wx.FULL_REPAINT_ON_RESIZE,
+            )
+        panel_settings = wx.ScrolledWindow(
+            self.notebook, 
+            wx.ID_ANY,
+            style=wx.TAB_TRAVERSAL | wx.FULL_REPAINT_ON_RESIZE
+            )
+        self.name = "app" 
+        self.__saved = True
+        self.__filename = None
         self.klass = "MyApp"
-        self.codegen_opt = 0 # if != 0, generates a separate file
-                             # for each class
+        self.codegen_opt = config.default_multiple_files
         def set_codegen_opt(value):
-            try: opt = int(value)
-            except ValueError: pass
-            else: self.codegen_opt = opt
+            try:
+                opt = int(value)
+            except ValueError:
+                pass
+            else:
+                self.codegen_opt = opt
         self.indent_mode = 1
-        self.indent_amount = 4
+        self.indent_amount = config.default_indent_amount
         def set_indent_mode(value):
             try: opt = int(value)
             except ValueError: pass
@@ -100,7 +164,7 @@ class Application(object):
         def get_output_path(): return os.path.expanduser(self.output_path)
         def set_output_path(value): self.output_path = value
         self.is_template = False
-        self.use_gettext = False
+        self.use_gettext = config.default_use_gettext
         def set_use_gettext(value): self.use_gettext = bool(int(value))
         self.for_version = wx.VERSION_STRING[:3]
         def set_for_version(value):
@@ -120,79 +184,135 @@ class Application(object):
             'for_version': (lambda : self.for_version, set_for_version),
             }
         self.name_prop = TextProperty(self, "name", panel, True)
+        self.name_prop.set_tooltip(
+            _('Name of the instance created from "Class"')
+            )
         self.klass_prop = TextProperty(self, "class", panel, True)
+        self.klass_prop.set_tooltip(
+            _("Name of the automatically generated class derived from wxApp")
+            )
 
         self.encoding = self._get_default_encoding()
         self.encoding_prop = TextProperty(self, 'encoding', panel)
+        self.encoding_prop.set_tooltip(
+            _("Encoding of the generated source files")
+            )
 
         self.use_gettext_prop = CheckBoxProperty(self, "use_gettext", panel,
                                                  _("Enable gettext support"))
+        self.use_gettext_prop.set_tooltip(
+            _("Enable internationalisation and localisation for the generated source files")
+            )
         TOP_WIN_ID = wx.NewId()
         self.top_win_prop = wx.Choice(panel, TOP_WIN_ID, choices=[],
                                      size=(1, -1))
+        self.top_win_prop.SetToolTip(wx.ToolTip(
+            _("This widget is used as top window in the wxApp start code")
+            ))
         self.top_window = '' # name of the top window of the generated app
 
-        
+        codegen_tooltips = [
+            _("Write all source code in one file"),
+            _("Split source code in one file per class / widget"),
+            ]
         self.codegen_prop = RadioProperty(self, "code_generation", panel,
                                           [_("Single file"),
                                            _("Separate file for" \
                                            " each class")],
-                                          label=_("Code Generation"))
+                                          label=_("Code Generation"),
+                                          tooltips=codegen_tooltips)
         self.indent_mode_prop = RadioProperty(self, "indent_mode",
-                                              lang_panels['C++'],
+                                              panel_settings,
                                               [_("Tabs"), _("Spaces")],
                                               columns=2,
                                               label=_("Indentation mode"))
         self.indent_amount_prop = SpinProperty(self, 'indent_amount',
-                                               lang_panels['C++'], r=(1, 100))
+                                               panel_settings, r=(1, 100))
+        self.indent_amount_prop.set_tooltip(
+            _('Number of spaces or tabs used for one indentation level.')
+            )
         self.source_ext_prop = TextProperty(self, 'source_ext',
-                                            lang_panels['C++'])
+                                            panel_settings)
+        self.source_ext_prop.set_tooltip(_('Extension of the source file'))
         self.header_ext_prop = TextProperty(self, 'header_ext',
-                                            lang_panels['C++'])
-        ext = getattr(common.code_writers.get('python'),
-                      'default_extensions', [])
-        wildcard = []
-        for e in ext:
-            wildcard.append('%s files (*.%s)|*.%s' % ('Python', e, e))
-        wildcard.append('All files|*')
-        dialog = FileDirDialog(self, panel, '|'.join(wildcard),
-                               _("Select output file"), _("Select output directory"),
-                               wx.SAVE|wx.OVERWRITE_PROMPT)
+                                            panel_settings)
+        self.header_ext_prop.set_tooltip(_('Extension of the header file'))
 
         _writers = common.code_writers.keys()
         columns = 3
 
         self.codewriters_prop = RadioProperty(self, "language", panel,
-                                              _writers, columns=columns)
-
+                                              _writers, columns=columns, 
+                                              sort=True, capitalize=True)
         self.codewriters_prop.set_str_value('python')
 
-        self.for_version_prop = RadioProperty(self, "for_version", panel,
-                                              ['2.6', '2.8'], columns=2,
-                                              label=_("wxWidgets compatibility"))
+        for_version_tooltips = [
+            _("Generate source files for wxWidgets version %s") % version \
+            for version in self.all_supported_versions
+            ]
+        self.for_version_prop = RadioProperty(
+            self,
+            "for_version",
+            panel,
+            self.all_supported_versions,
+            columns=2,
+            label=_("wxWidgets compatibility"),
+            tooltips=for_version_tooltips,
+            )
         self.for_version_prop.set_str_value(self.for_version)
         
-        # ALB 2004-01-18
         self.access_functions['use_new_namespace'] = (
             self.get_use_old_namespace, self.set_use_old_namespace)
         self.use_old_namespace_prop = CheckBoxProperty(
-            self, 'use_new_namespace', lang_panels['python'],
-            _('Use old "from wxPython.wx"\nimport'))
-        
-        # `overwrite' property - added 2003-07-15
-        self.overwrite = False
-        def get_overwrite(): return self.overwrite
-        def set_overwrite(val): self.overwrite = bool(int(val))
-        self.access_functions['overwrite'] = (get_overwrite, set_overwrite)
-        self.overwrite_prop = CheckBoxProperty(self, 'overwrite', panel,
-                                               _('Overwrite existing sources'))
+            self, 'use_new_namespace', panel_settings,
+            _('Use old import\n"from wxPython.wx import *"'))
+        self.use_old_namespace_prop.set_tooltip(
+            _('It is generally recommended to use the new namespace. '
+              'The old one ("from wxPython.wx import *") has some '
+              'significant drawbacks like potential namespace conflicts.'
+              ))
 
+        self.overwrite = config.default_overwrite
+        self.access_functions['overwrite'] = \
+            (self.get_overwrite, self.set_overwrite)
+        self.overwrite_prop = CheckBoxProperty(
+            self,
+            'overwrite',
+            panel,
+            _('Overwrite existing sources'),
+            )
+        self.overwrite_prop.set_tooltip(
+            _("Overwrite existing source files or modify the code sequences "
+              "generated by wxGlade in place")
+            )
+            
+        dialog = FileDirDialog(
+            self,
+            _('All files|*'),
+            _("Select output file"),
+            _("Select output directory"),
+            wx.SAVE|wx.OVERWRITE_PROMPT
+            )
         self.outpath_prop = DialogProperty(self, "output_path", panel,
                                            dialog, label=_('Output path'))
+        # update wildcards and default extention in the dialog
+        self._update_dialog(self.outpath_prop.dialog, 'python')
+        self.outpath_prop.set_tooltip(
+            _("Output file or directory")
+            )
+
         BTN_ID = wx.NewId()
-        btn = wx.Button(panel, BTN_ID, _("Generate code"))
+        btn = wx.Button(
+            panel,
+            BTN_ID,
+            _("Generate code"),
+            name="BtnGenerateCode",
+            )
+        btn.SetToolTip(wx.ToolTip(_("Start generating source files")))
+
+        # layout of self.notebook - page "Application"
+        #=============================================
 
-        # layout of self.notebook
         sizer = wx.BoxSizer(wx.VERTICAL)
         sizer.Add(self.name_prop.panel, 0, wx.EXPAND)
         sizer.Add(self.klass_prop.panel, 0, wx.EXPAND)
@@ -201,7 +321,9 @@ class Application(object):
         szr = wx.BoxSizer(wx.HORIZONTAL)
         from widget_properties import _label_initial_width as _w
         label = wx.StaticText(panel, -1, _("Top window"), size=(_w, -1))
-        label.SetToolTip(wx.ToolTip(_("Top window")))
+        label.SetToolTip(wx.ToolTip(
+            _("This widget is used as top window in the wxApp start code")
+            ))
         szr.Add(label, 2, wx.ALL|wx.ALIGN_CENTER, 3)
         szr.Add(self.top_win_prop, 5, wx.ALL|wx.ALIGN_CENTER, 3)
         sizer.Add(szr, 0, wx.EXPAND)
@@ -213,36 +335,53 @@ class Application(object):
                 
         sizer.Add(btn, 0, wx.ALL|wx.EXPAND, 5)
 
-        def do_add(l, p, s):
-            p.SetAutoLayout(True)
-            p.SetSizer(s)
-            s.Layout()
-            s.Fit(p)
-            h = p.GetSize()[1]
-            self.notebook.AddPage(p, l)
-            p.SetScrollbars(1, 5, 1, int(math.ceil(h/5.0)))
-            
-        do_add(_('Application'), panel, sizer)
+        self._add_page(_('Application'), panel, sizer)
+        
+        # layout self.notebook - page "Settings"
+        #=======================================
 
-        # lang_panels['python']
+        # general settings
+        staticbox_general = wx.StaticBox(
+            panel_settings,
+            wx.ID_ANY,
+            _("General Settings"),
+            )
+        sizer_general = wx.StaticBoxSizer(staticbox_general, wx.VERTICAL)
+        sizer_general.Add(self.indent_mode_prop.panel, 0, wx.ALL|wx.EXPAND, 4)
+        sizer_general.Add(self.indent_amount_prop.panel, 0, wx.EXPAND)
+
+        # python specific settings
+        staticbox_python = wx.StaticBox(
+            panel_settings,
+            wx.ID_ANY,
+            _("Python Settings"),
+            )
+        sizer_python = wx.StaticBoxSizer(staticbox_python, wx.VERTICAL)
+        sizer_python.Add(self.use_old_namespace_prop.panel, 0, wx.EXPAND)
+
+        # C++ specific settings
+        staticbox_cpp = wx.StaticBox(
+        panel_settings,
+            wx.ID_ANY,
+            _("C++ Settings"),
+            )
+        sizer_cpp = wx.StaticBoxSizer(staticbox_cpp, wx.VERTICAL)
+        sizer_cpp.Add(self.source_ext_prop.panel, 0, wx.EXPAND)
+        sizer_cpp.Add(self.header_ext_prop.panel, 0, wx.EXPAND)
+
+        # add all to one sizer
         sizer = wx.BoxSizer(wx.VERTICAL)
-        sizer.Add(self.use_old_namespace_prop.panel, 0, wx.EXPAND)
-        do_add('Python', lang_panels['python'], sizer)
-
-        # lang_panels['C++']
-        sizer = wx.BoxSizer(wx.VERTICAL)
-        sizer.Add(self.indent_mode_prop.panel, 0, wx.ALL|wx.EXPAND, 4)
-        sizer.Add(self.indent_amount_prop.panel, 0, wx.EXPAND)
-        sizer.Add(self.source_ext_prop.panel, 0, wx.EXPAND)
-        sizer.Add(self.header_ext_prop.panel, 0, wx.EXPAND)
-        do_add('C++', lang_panels['C++'], sizer)
-
+        sizer.Add(sizer_general, 0, wx.EXPAND|wx.ALL, 3)
+        sizer.Add(sizer_python,  0, wx.EXPAND|wx.ALL, 3)
+        sizer.Add(sizer_cpp,     0, wx.EXPAND|wx.ALL, 3)
+        self._add_page(_('Settings'), panel_settings, sizer)
+        
         wx.EVT_BUTTON(btn, BTN_ID, self.generate_code)
         wx.EVT_CHOICE(self.top_win_prop, TOP_WIN_ID, self.set_top_window)
 
         # this is here to keep the interface similar to the various widgets
         # (to simplify Tree)
-        self.widget = None # this is always None
+        self.widget = None  # this is always None
 
     def set_name(self, value):
         value = "%s" % value
@@ -260,14 +399,57 @@ class Application(object):
             self.klass = value
     set_klass_pattern = re.compile('^[a-zA-Z]+[\w:.0-9-]*$')
 
+    def _add_page(self, label, page, sizer):
+        """\
+        Add a page to properties notebook (L{self.notebook})
+        """
+        page.SetAutoLayout(True)
+        page.SetSizer(sizer)
+        sizer.Layout()
+        sizer.Fit(page)
+        self.notebook.AddPage(page, label)
+        h = page.GetSize()[1]
+        page.SetScrollbars(1, 5, 1, int(math.ceil(h/5.0)))
+
     def _get_default_encoding(self):
         """\
-        Returns the name of the default character encoding of this machine
+        Return determinated machine default character encoding.
+       
+        The default application L{config.default_encoding} is the fallback
+        only.
         """
-        import locale
-        locale.setlocale(locale.LC_ALL)
-        try: return locale.nl_langinfo(locale.CODESET)
-        except AttributeError: return 'ISO-8859-15' # this is what I use...
+        # try to set locale
+        try:
+            locale.setlocale(locale.LC_ALL)
+        except locale.Error:
+            # ignore problems by fallback to ascii
+            print 'WARNING: Setting locale failed. Use "ascii" instead'
+            return 'ascii'
+
+        # try to query character encoding used in the selected locale
+        try:
+            encoding = locale.nl_langinfo(locale.CODESET)
+        except AttributeError, e:
+            print 'WARNING: locale.nl_langinfo(locale.CODESET) failed: %s' % str(e)
+            # try getdefaultlocale, it used environment variables
+            try:
+                encoding = locale.getdefaultlocale()[1]
+            except ValueError:
+                encoding = config.default_encoding
+
+        # On Mac OS X encoding may None or '' somehow
+        if not encoding:
+            encoding = config.default_encoding
+            print 'WARNING: Empty encoding. Use "%s" instead' % encoding
+
+        # check if a codec for the encoding exists
+        try:
+            codecs.lookup(encoding)
+        except LookupError:
+            print 'WARNING: No codec for encoding "%s" found. Use "ascii" instead' % encoding
+            encoding = 'ascii'
+            
+        return encoding.upper()
 
     def get_encoding(self):
         return self.encoding
@@ -282,13 +464,10 @@ class Application(object):
 
     def set_language(self, value):
         language = self.codewriters_prop.get_str_value()
-        ext = getattr(common.code_writers[language], 'default_extensions', [])
-        wildcard = []
-        for e in ext:
-            wildcard.append(_('%s files (*.%s)|*.%s') % (language.capitalize(),
-                                                      e, e))
-        wildcard.append(_('All files|*'))
-        self.outpath_prop.dialog.set_wildcard('|'.join(wildcard))
+        
+        # update wildcards and default extention in the dialog
+        self._update_dialog(self.outpath_prop.dialog, language)
+        
         # check that the new language supports all the widgets in the tree
         if self.language != language:
             self.language = language
@@ -307,7 +486,9 @@ class Application(object):
                 if t[0] == '*': common.app_tree.set_title(t[1:].strip())
     saved = property(_get_saved, _set_saved)
 
-    def _get_filename(self): return self.__filename
+    def _get_filename(self):
+        return self.__filename
+
     def _set_filename(self, value):
         if not misc.streq(self.__filename, value):
             self.__filename = value
@@ -318,7 +499,13 @@ class Application(object):
             else:
                 common.app_tree.set_title(flag)
     filename = property(_get_filename, _set_filename)
-       
+
+    def get_overwrite(self):
+        return self.overwrite
+
+    def set_overwrite(self, val):
+        self.overwrite = bool(int(val))
+
     def get_top_window(self): return self.top_window
 
     def set_top_window(self, *args):
@@ -365,14 +552,17 @@ class Application(object):
         """
         self.klass = "MyApp"; self.klass_prop.set_value("MyApp")
         self.klass_prop.toggle_active(False)
-        self.name = "app"; self.name_prop.set_value("app")
+        self.name = "app"
+        self.name_prop.set_value("app")
         self.name_prop.toggle_active(False)
-        self.codegen_opt = 0; self.codegen_prop.set_value(0)
+        self.codegen_opt = config.default_multiple_files
+        self.codegen_prop.set_value(config.default_multiple_files)
         self.indent_mode = 1
-        self.indent_amount = 4
+        self.indent_amount = config.default_indent_amount
         self.cpp_source_ext = 'cpp'
         self.cpp_header_ext = 'h'
-        self.output_path = ""; self.outpath_prop.set_value("")
+        self.output_path = ""
+        self.outpath_prop.set_value("")
         # do not reset language, but call set_language anyway to update the
         # wildcard of the file dialog
         self.set_language(self.get_language())
@@ -447,38 +637,67 @@ class Application(object):
                 self.overwrite = True
             class_names = common.app_tree.write(out) # write the xml onto a
                                                      # temporary buffer
-            if not os.path.isabs(self.output_path) and \
-               self.filename is not None:
+            if not os.path.isabs(self.output_path) and self.filename:
                 out_path = os.path.join(os.path.dirname(self.filename),
                                         self.output_path)
             else:
                 out_path = None
-            CodeWriter(common.code_writers[cw], out.getvalue(), True,
-                       preview=preview, out_path=out_path,
-                       class_names=class_names)
+            CodeWriter(
+                common.code_writers[cw],
+                out.getvalue(), True,
+                preview=preview,
+                out_path=out_path,
+                class_names=class_names,
+                )
             if preview and cw == 'python':
                 common.code_writers[cw].use_new_namespace = old
                 self.overwrite = overwrite
+        except (errors.WxgOutputDirectoryNotExist,
+                errors.WxgOutputDirectoryNotWritable,
+                errors.WxgOutputPathIsDirectory,
+                ), inst:
+            wx.MessageBox(
+                _("Error generating code:\n%s") % inst,
+                _("Error"),
+                wx.OK|wx.CENTRE|wx.ICON_ERROR,
+                )
         except (IOError, OSError), msg:
-            wx.MessageBox(_("Error generating code:\n%s") % msg, _("Error"),
-                         wx.OK|wx.CENTRE|wx.ICON_ERROR)
+            wx.MessageBox(
+                _("Error generating code:\n%s") % msg,
+                _("Error"),
+                wx.OK|wx.CENTRE|wx.ICON_ERROR,
+                )
         except Exception, msg:
-            traceback.print_exc()
-            wx.MessageBox(_("An exception occurred while generating the code "
-                         "for the application.\n"
-                         "This is the error message associated with it:\n"
-                         "        %s\n"
-                         "For more details, look at the full traceback "
-                         "on the console.\nIf you think this is a wxGlade bug,"
-                         " please report it.") % msg, _("Error"),
-                         wx.OK|wx.CENTRE|wx.ICON_ERROR)
+            common.message.exception(_('Internal Error'))
+            wx.MessageBox(
+                _("An exception occurred while generating the code "
+                  "for the application.\n"
+                  "This is the error message associated with it:\n"
+                  "        %s\n"
+                  "For more details, look at the full traceback "
+                  "on the console.\nIf you think this is a wxGlade bug,"
+                  " please report it.") % msg,
+                  _("Error"),
+                  wx.OK|wx.CENTRE|wx.ICON_ERROR,
+                  )
         else:
             if not preview:
-                wx.MessageBox(_("Code generation completed successfully"),
-                             _("Information"), wx.OK|wx.CENTRE|wx.ICON_INFORMATION)
+                if config.preferences.show_completion:
+                    # Show informational dialog
+                    wx.MessageBox(
+                        _("Code generation completed successfully"),
+                        _("Information"),
+                        wx.OK | wx.CENTRE | wx.ICON_INFORMATION,
+                        )
+                else:
+                    # Show message in application status bar
+                    app = wx.GetApp()
+                    frame = app.GetTopWindow()
+                    frame.user_message(_('Code generated'))
 
     def get_name(self):
-        if self.name_prop.is_active(): return self.name
+        if self.name_prop.is_active():
+            return self.name
         return ''
 
     def get_class(self):
@@ -489,13 +708,13 @@ class Application(object):
 
     def is_visible(self): return True
 
-    def preview(self, widget, out_name=[None]):
-        if out_name[0] is None:
+    def preview(self, widget, out_name=None):
+        if out_name is None:
             import warnings
             warnings.filterwarnings("ignore", "tempnam", RuntimeWarning,
                                     "application")
-            out_name[0] = os.tempnam(None, 'wxg') + '.py'
-            #print 'Temporary name:', out_name[0]
+            out_name = os.tempnam(None, 'wxg') + '.py'
+            #print 'Temporary name:', out_name
         widget_class_name = widget.klass
 
         # make a valid name for the class (this can be invalid for
@@ -511,7 +730,7 @@ class Application(object):
                        (random.randrange(10**8, 10**9), widget.klass)
             
         self.real_output_path = self.output_path
-        self.output_path = out_name[0]
+        self.output_path = out_name
         real_codegen_opt = self.codegen_opt
         real_language = self.language
         real_use_gettext = self.use_gettext
@@ -565,7 +784,7 @@ class Application(object):
                 if os.path.isfile(name):
                     os.unlink(name)
         except Exception, e:
-            #traceback.print_exc()
+            #common.message.exception(_('Internal Error'))
             widget.preview_widget = None
             widget.preview_button.SetLabel(_('Preview'))
             wx.MessageBox(_("Problem previewing gui: %s") % str(e), _("Error"),
@@ -581,11 +800,12 @@ class Application(object):
         return frame
 
     def get_use_old_namespace(self):
-        try: return not common.code_writers['python'].use_new_namespace
-        except: return False
+        try:
+            return not common.code_writers['python'].use_new_namespace
+        except:
+            return False
 
     def set_use_old_namespace(self, val):
-        #print "set use old namespace"
         try:
             common.code_writers['python'].use_new_namespace = not bool(int(val))
         except:
@@ -608,10 +828,10 @@ class Application(object):
                 ok = xrcgen.obj_builders.get(cname, None) is not \
                      xrcgen.NotImplementedXrcObject
             if not ok:
-                common.message(_('WARNING'),
+                common.message.warn(
                                _('No %s code generator for %s (of type %s)'
                                ' available'),
-                               language.capitalize(), widget.name, cname)
+                               misc.capitalize(language), widget.name, cname)
         else:
             # in this case, we check all the widgets in the tree
             def check_rec(node):
@@ -624,4 +844,23 @@ class Application(object):
                 for c in common.app_tree.root.children:
                     check_rec(c)
 
+    def _update_dialog(self, dialog, language):
+        """\
+        Update wildcards and default extension in the generic file and
+        directory dialog (L{FileDirDialog}).
+        """
+        ext = getattr(common.code_writers[language], 'default_extensions', [])
+        wildcards = []
+        for e in ext:
+            wildcards.append(
+                _('%s files (*.%s)|*.%s') % (misc.capitalize(language), e, e)
+                )
+        wildcards.append(_('All files|*'))
+        dialog.wildcard = '|'.join(wildcards)
+        
+        if ext:
+            dialog.default_extension = '.%s' % ext[0]
+        else:
+            dialog.default_extension = None
+
 # end of class Application
