@@ -1,11 +1,13 @@
-# common.py: global variables
-#
-# Copyright (c) 2002-2007 Alberto Griggio <agriggio@users.sourceforge.net>
-# License: MIT (see license.txt)
-# THIS PROGRAM COMES WITH NO WARRANTY
+"""
+Global variables
+
+@copyright: 2002-2007 Alberto Griggio <agriggio@users.sourceforge.net>
+@license: MIT (see license.txt) - THIS PROGRAM COMES WITH NO WARRANTY
+"""
 
 import os
 import sys
+import traceback
 
 use_gui = True
 """\
@@ -13,7 +15,7 @@ If False, the program is invoked from the command-line in "batch" mode
 (for code generation only)
 """
 
-nohg_version = '0.6.5'
+nohg_version = '0.6.7'
 """\
 Version number to return if no hg repo has been found
 """
@@ -30,48 +32,47 @@ def _get_version():
     @rtype: String
     @see: L{nohg_version}
     """
-    main_version = ''
+#    main_version = ''
+    main_version = nohg_version
     repo_changed = []
-    try:
-        from mercurial.hg import repository
-        from mercurial.ui import ui
-        from mercurial.node import short
-        from mercurial.error import RepoError
-    except ImportError:
-        # no mercurial module available
-        main_version = nohg_version
-    except:
-        # unkown failure
-        main_version = nohg_version
-    else:
-        # try to open local hg repository
-        try:
-            repo = repository(ui(), os.path.dirname(__file__))
-        except RepoError:
-            # no mercurial repository found
-            main_version = nohg_version
-        else:
-            ctx = repo[None]
-            parents = ctx.parents()
-            repo_changed = ctx.files() + ctx.deleted()
-            if len(parents) == 1 and not repo_changed:
-                # release tag isn't at tip it's -2 (one below tip)
-                parents = parents[0].parents()
-                node = parents[0].node()
-                tags = repo.nodetags(node)
-                # look for the special 'rel_X.X' tag
-                for tag in tags:
-                    if tag.startswith('rel_') and len(tag) > 4:
-                        main_version = tag[4:]
-                        break
-                # handle untagged version e.g. tip
-                if not main_version:
-                    main_version = short(node)
-            else:
-                main_version = '%s%s' % (
-                    '+'.join([short(p.node()) for p in parents]),
-                    repo_changed and '+' or ''
-                    )
+#    try:
+#        from mercurial.hg import repository
+#        from mercurial.ui import ui
+#        from mercurial.node import short
+#        from mercurial.error import RepoError
+#    except ImportError:
+#        # no mercurial module available
+#        main_version = nohg_version
+#    except:
+#        # unkown failure
+#        main_version = nohg_version
+#    else:
+#        # try to open local hg repository
+#        try:
+#            repo = repository(ui(), os.path.dirname(__file__))
+#        except RepoError:
+#            # no mercurial repository found
+#            main_version = nohg_version
+#        else:
+#            ctx = repo[None]
+#            parents = ctx.parents()
+#            repo_changed = ctx.files() + ctx.deleted()
+#            if len(parents) == 1 and not repo_changed:
+#                # release tag isn't at tip it's -2 (one below tip)
+#                parents = parents[0].parents()
+#                node = parents[0].node()
+#                tags = repo.nodetags(node)
+#                # look for the special 'rel_X.X' tag
+#                for tag in tags:
+#                    if tag.startswith('rel_') and len(tag) > 4:
+#                        main_version = tag[4:]
+#                        break
+#                # handle untagged version e.g. tip
+#                if not main_version:
+#                    main_version = short(node)
+#            else:
+#                main_version = '%s' % \
+#                    '+'.join([short(p.node()) for p in parents])
 
     suffix_changed = repo_changed and '+' or ''
     suffix_edition = hasattr(sys, 'frozen') \
@@ -92,6 +93,11 @@ py_version = sys.version.split()[0]
 Python version
 """
 
+platform = None
+"""\
+Current platform (mostly wx.Platform)
+"""
+
 wxglade_path = '.'
 """\
 Program path, set in wxglade.py
@@ -216,6 +222,7 @@ Dictionary of objects used to generate the code in a given language.
 @note: A code writer object must implement this interface:
  - initialize(out_path, multi_files)
  - language
+ - setup
  - add_widget_handler(widget_name, handler[, properties_handler])
  - add_property_handler(property_name, handler[, widget_name])
  - add_object(top_obj, sub_obj)
@@ -234,23 +241,27 @@ def load_code_writers():
     codegen_path = os.path.join(wxglade_path, 'codegen')
     sys.path.insert(0, codegen_path)
     for module in os.listdir(codegen_path):
-        name = os.path.splitext(module)[0]
+        name, ext = os.path.splitext(module)
         # skip __init__
         if name == "__init__":
             continue
         # allow regular files only
         if not os.path.isfile(os.path.join(codegen_path, module)):
             continue
+        # ignore none python files
+        if ext not in ['.py', '.pyo', '.pyc']:
+            continue
         # skip already imported modules
         if name in sys.modules:
             continue
         # import file and initiate code writer
         try:
             writer = __import__(name).writer
-        except (ImportError, AttributeError, ValueError):
-            if use_gui:
-                print _('"%s" is not a valid code generator module') % \
-                      module
+        except (AttributeError, ImportError, NameError, 
+                SyntaxError, ValueError):
+            message.exception(
+                _('"%s" is not a valid code generator module') % module
+                )
         else:
             code_writers[writer.language] = writer
             if hasattr(writer, 'setup'):
@@ -306,11 +317,9 @@ def __load_widgets(widget_dir):
                         sys.path.pop()
                 else:
                     raise
-        except (ImportError, AttributeError):
-            if use_gui:
-                print _('ERROR loading "%s"') % module
-                import traceback
-                traceback.print_exc()
+        except (AttributeError, ImportError, NameError, 
+                SyntaxError, ValueError):
+            message.exception(_('ERROR loading "%s"') % module)
         else:
             if use_gui:
                 print '\t' + module
@@ -433,7 +442,7 @@ def save_file(filename, content, which='wxg'):
     """\
     Save I{content} to file named I{filename} and, if user's preferences say
     so and I{filename} exists, makes a backup copy of it.
-    
+
     @note: Exceptions that may occur while performing the operations are not
            handled.
 
@@ -451,7 +460,9 @@ def save_file(filename, content, which='wxg'):
             'Unknown value "%s" for parameter "which"!' % which
             )
     try:
-        if do_backup and filename not in _backed_up and os.path.isfile(filename):
+        if do_backup and \
+           filename not in _backed_up and \
+           os.path.isfile(filename):
             # make a backup copy of filename
             infile = open(filename)
             outfile = open(filename + config.preferences.backup_suffix, 'w')
@@ -468,7 +479,7 @@ def save_file(filename, content, which='wxg'):
         if savecontent:
             directory = os.path.dirname(filename)
             if directory and not os.path.isdir(directory):
-                os.mkdir(directory)
+                os.makedirs(directory)
             outfile = open(filename, 'w')
             outfile.write(content)
             outfile.close()
@@ -562,12 +573,27 @@ def generated_from():
 
 
 class MessageLogger(object):
+    """\
+    Small own logging facility
+    
+    @ivar disabled: If True log messages won't be processed
+    @type disabled: Boolean
+    
+    @ivar lines: Message buffer
+    @type lines: List of strings
+    
+    @ivar logger: Reference to an instance of L{msgdialog.MessageDialog}
+    """
+
     def __init__(self):
         self.disabled = False
         self.lines = []
         self.logger = None
 
     def _setup_logger(self):
+        """\
+        Create L{msgdialog.MessageDialog} instance.
+        """
         import msgdialog
         self.logger = msgdialog.MessageDialog(None, -1, "")
         self.logger.msg_list.InsertColumn(0, "")
@@ -583,18 +609,51 @@ class MessageLogger(object):
             else:
                 msg = misc.wxstr(fmt)
             self.lines.extend(msg.splitlines())
-##             if kind == 'WARNING':
-##                 wx.LogWarning(msg)
-##             else:
-##                 wx.LogMessage(msg)
-        else:
+
+        # show errors and exceptions always at console
+        if not use_gui or kind in [_("EXCEPTION"),  _("ERROR")]:
             if args:
                 msg = fmt % tuple(args)
             else:
                 msg = fmt
             print "%s: %s" % (kind, msg)
 
+    def debug(self, fmt, *args):
+        """\
+        Show debug messages
+        """
+        self.__call__(_("DEBUG"), fmt, *args)
+
+    def info(self, fmt, *args):
+        """\
+        Show informational messages
+        """
+        self.__call__(_("INFO"), fmt, *args)
+
+    def warn(self, fmt, *args):
+        """\
+        Show warning messages
+        """
+        self.__call__(_("WARNING"), fmt, *args)
+
+    def error(self, fmt, *args):
+        """\
+        Show error  messages
+        """
+        self.__call__(_("ERROR"), fmt, *args)
+        
+    def exception(self, fmt, *args):
+        """\
+        Show exception details
+        """
+        self.__call__(_("EXCEPTION"), fmt, *args)
+        self.__call__(_("EXCEPTION"), _("Exception details:"))
+        self.__call__(_("EXCEPTION"), traceback.format_exc())
+
     def flush(self):
+        """\
+        Show log messages if L{use_gui} is True
+        """
         if self.lines and use_gui:
             if not self.logger:
                 self._setup_logger()
@@ -610,3 +669,6 @@ class MessageLogger(object):
 # end of class MessageLogger
 
 message = MessageLogger()
+"""\
+L{MessageLogger} instance
+"""
