@@ -1,24 +1,26 @@
-# main.py: Main wxGlade module: defines wxGladeFrame which contains the buttons
-# to add widgets and initializes all the stuff (tree, property_frame, etc.)
-# 
-# Copyright (c) 2002-2007 Alberto Griggio <agriggio@users.sourceforge.net>
-#
-# License: MIT (see license.txt)
-# THIS PROGRAM COMES WITH NO WARRANTY
+"""
+Main wxGlade module: defines wxGladeFrame which contains the buttons to add
+widgets and initializes all the stuff (tree, property_frame, etc.)
 
+@copyright: 2002-2007 Alberto Griggio <agriggio@users.sourceforge.net>
+@license: MIT (see license.txt) - THIS PROGRAM COMES WITH NO WARRANTY
+"""
+
+# import general python modules
+import os
+import os.path
 import sys
-
-# ensure minimal wx version
-if not hasattr(sys, 'frozen'):
-    import wxversion
-    wxversion.ensureMinimal("2.6")
-
 import wx
+
+# import project modules
 from widget_properties import *
 from tree import WidgetTree
-import common, os, os.path, misc, config
+import common
+import application
+import misc
+import config
+import configdialog
 import clipboard
-
 import template
 
 
@@ -64,8 +66,10 @@ class ToggleBoxEvent(wx.PyCommandEvent):
 
 
 class ToggleButtonBox(wx.Panel):
-    def __init__(self, parent, id, choices=[], value=0):
+    def __init__(self, parent, id, choices=None, value=0):
         wx.Panel.__init__(self, parent, id)
+        if choices is None:
+            choices = []
         self.buttons = [wx.ToggleButton(self, -1, c) for c in choices]
         self.selected = None
         self.SetValue(value)
@@ -131,6 +135,10 @@ class wxGladeArtProvider(wx.ArtProvider):
 class wxGladeFrame(wx.Frame):
     """\
     Main frame of wxGlade (palette)
+    
+    @ivar cur_dir: Last visited directory, used for wxFileDialog and not
+                   for KDE dialogs
+    @type cur_dir: String
     """
     def __init__(self, parent=None):
         style = wx.SYSTEM_MENU|wx.CAPTION|wx.MINIMIZE_BOX|wx.RESIZE_BORDER
@@ -267,12 +275,7 @@ class wxGladeFrame(wx.Frame):
         wx.EVT_MENU(self, RELOAD_ID, self.reload_app)
 
         PREVIEW_ID = wx.NewId()
-        def preview(event):
-            if common.app_tree.cur_widget is not None:
-                p = misc.get_toplevel_widget(common.app_tree.cur_widget)
-                if p is not None:
-                    p.preview(None)
-        wx.EVT_MENU(self, PREVIEW_ID, preview)
+        wx.EVT_MENU(self, PREVIEW_ID, self.preview)
 
         self.accel_table = wx.AcceleratorTable([
             (wx.ACCEL_CTRL, ord('N'), NEW_ID),
@@ -394,7 +397,6 @@ class wxGladeFrame(wx.Frame):
                                   style=frame_style)
         self.tree_frame.SetIcon(icon)
         
-        import application
         app = application.Application(common.property_panel)
         common.app_tree = WidgetTree(self.tree_frame, app)
         self.tree_frame.SetSize((300, 300))
@@ -506,7 +508,34 @@ class wxGladeFrame(wx.Frame):
             self.user_message(_("Auto saving... done"))
         
     def edit_preferences(self, event):
-        config.edit_preferences()
+        dialog = configdialog.wxGladePreferences(config.preferences)
+        if dialog.ShowModal() == wx.ID_OK:
+            wx.MessageBox(_('Changes will take effect after wxGlade is restarted'),
+                          _('Preferences saved'),
+                          wx.OK|wx.CENTRE|wx.ICON_INFORMATION)
+            dialog.set_preferences()
+        dialog.Destroy()
+        
+    def preview(self, event):
+        """\
+        Generate preview of the current loaded project.
+        
+        A preview can be triggered by keyboard shortcut or by pressing the
+        preview button. The preview can be triggered for all selected widgets.
+        This doesn't mean that the widget is opened for editing.
+        
+        A good indicator for editing is the availability of the preview
+        button.
+       
+        @see: L{edit_windows.PreviewMixin.preview_button}
+        """
+        # Show preview only, if preview_button is available
+        #
+        if not getattr(common.app_tree.cur_widget, 'preview_button', None):
+            return
+        preview_widget = misc.get_toplevel_widget(common.app_tree.cur_widget)
+        if preview_widget is not None:
+            preview_widget.preview(None)
 
     def show_tree(self, event):
         self.tree_frame.Show()
@@ -640,9 +669,11 @@ class wxGladeFrame(wx.Frame):
             if not is_filelike:
                 os.chdir(os.path.dirname(infilename))
                 infile = open(infilename)
+                print _('Read wxGlade project from file "%s"') % infilename
             else:
                 infile = infilename
                 infilename = getattr(infile, 'name', None)
+                print _('Read wxGlade project from file-like object')
             if use_progress_dialog and config.preferences.show_progress:
                 p = ProgressXmlWidgetBuilder(input_file=infile)
             else:
@@ -661,22 +692,28 @@ class wxGladeFrame(wx.Frame):
             os.chdir(old_dir)
             return False            
         except Exception, msg:
-            import traceback; traceback.print_exc()
+            common.message.exception(
+                _('An exception occurred while loading file "%s".') % \
+                    infilename.encode('ascii', 'replace')
+                )
 
-            if locals().has_key('infile') and not is_filelike: infile.close()
+            if locals().has_key('infile') and not is_filelike:
+                infile.close()
             common.app_tree.clear()
             common.property_panel.Reparent(self.frame2)
             common.app_tree.app.saved = True
-            wx.MessageBox(_("An exception occurred while loading file \"%s\".\n"
-                            "This is the error message associated with it:\n"
-                            "        %s\n"
-                            "For more details, look at the full traceback "
-                            "on the console.\n"
-                            "If you think this is a wxGlade bug,"
-                            " please report it.") % \
-                          (misc.wxstr(infilename), misc.wxstr(msg)),
-                          _("Error"),
-                          wx.OK|wx.CENTRE|wx.ICON_ERROR)
+            wx.MessageBox(
+                _("An exception occurred while loading file \"%s\".\n"
+                  "This is the error message associated with it:\n"
+                  "        %s\n"
+                  "For more details, look at the full traceback "
+                  "on the console.\n"
+                  "If you think this is a wxGlade bug,"
+                  " please report it.") % \
+                  (misc.wxstr(infilename), misc.wxstr(msg)),
+                _("Error"),
+                wx.OK|wx.CENTRE|wx.ICON_ERROR
+                )
             # reset the auto-expansion of nodes
             common.app_tree.auto_expand = True
             os.chdir(old_dir)
@@ -717,11 +754,13 @@ class wxGladeFrame(wx.Frame):
         """\
         saves a wxGlade project onto an xml file
         """
-        if not common.app_tree.app.filename or common.app_tree.app.is_template:
+        if not common.app_tree.app.filename or \
+           common.app_tree.app.is_template:
             self.save_app_as(event)
         else:
             # check whether we are saving a template
-            if os.path.splitext(common.app_tree.app.filename)[1] == ".wgt":
+            ext = os.path.splitext(common.app_tree.app.filename)[1].lower()
+            if ext == ".wgt":
                 common.app_tree.app.is_template = True
             self._save_app(common.app_tree.app.filename)
 
@@ -738,7 +777,10 @@ class wxGladeFrame(wx.Frame):
             wx.MessageBox(_("Error saving app:\n%s") % msg, _("Error"),
                          wx.OK|wx.CENTRE|wx.ICON_ERROR)
         except Exception, msg:
-            import traceback; traceback.print_exc()
+            common.message.exception(
+                _('An exception occurred while saving file "%s".') % \
+                    fn.encode('ascii', 'replace')
+                )
             common.app_tree.app.saved = False
             fn = filename
             wx.MessageBox(_("An exception occurred while saving file "
@@ -748,8 +790,10 @@ class wxGladeFrame(wx.Frame):
                             "For more details, look at the full traceback "
                             "on the console.\nIf you think this is a "
                             "wxGlade bug,"
-                            " please report it.") % (fn, msg), _("Error"),
-                          wx.OK|wx.CENTRE|wx.ICON_ERROR)
+                            " please report it.") % (fn, msg),
+                          _("Error"),
+                          wx.OK|wx.CENTRE|wx.ICON_ERROR
+                          )
         else:
             common.app_tree.app.saved = True
             common.remove_autosaved() # ALB 2004-10-15
@@ -763,6 +807,8 @@ class wxGladeFrame(wx.Frame):
         """\
         saves a wxGlade project onto an xml file chosen by the user
         """
+        # wx.SAVE & wx.OVERWRITE_PROMPT are wx2.6
+        # both flags occurs several times
         fn = misc.FileSelector(_("Save project as..."),
                                wildcard="wxGlade files (*.wxg)|*.wxg|"
                                "wxGlade Template files (*.wgt) |*.wgt|"
@@ -770,6 +816,11 @@ class wxGladeFrame(wx.Frame):
                                flags=wx.SAVE|wx.OVERWRITE_PROMPT,
                                default_path=self.cur_dir)
         if fn:
+            # check for file extension and add default extension if missing
+            ext = os.path.splitext(fn)[1].lower()
+            if not ext:
+                fn = "%s.wxg" % fn
+                
             common.app_tree.app.filename = fn
             #remove the template flag so we can save the file.
             common.app_tree.app.is_template = False
@@ -779,6 +830,9 @@ class wxGladeFrame(wx.Frame):
             self.file_history.AddFileToHistory(fn)
 
     def save_app_as_template(self, event):
+        """\
+        save a wxGlade project as a template
+        """
         data = getattr(common.app_tree.app, 'template_data', None)
         outfile, data = template.save_template(data)
         if outfile:
@@ -812,12 +866,20 @@ class wxGladeFrame(wx.Frame):
             wx.CallAfter(wx.GetApp().ExitMainLoop)
 
     def show_about_box(self, event):
+        """\
+        show the about dialog
+        
+        @see: L{about.wxGladeAboutBox}
+        """
         if self.about_box is None:
             import about
             self.about_box = about.wxGladeAboutBox(None)
         self.about_box.ShowModal()
 
     def show_tutorial(self, event):
+        """\
+        show the wxGlade user manual
+        """
         if wx.Platform == "__WXMAC__":
             os.system('open -a Help\ Viewer.app %s' % common.tutorial_file)
         else:
@@ -847,28 +909,34 @@ class wxGladeFrame(wx.Frame):
         if not self.ask_save():
             return
         
-        infile = misc.FileSelector(_("Import file"),
+        infilename = misc.FileSelector(_("Import file"),
                                    wildcard="XRC files (*.xrc)"
                                    "|*.xrc|All files|*",
                                    flags=wx.OPEN|wx.FILE_MUST_EXIST,
                                    default_path=self.cur_dir)
-        if infile:
+        if infilename:
             buf = cStringIO.StringIO()
             try:
-                xrc2wxg.convert(infile, buf)
+                xrc2wxg.convert(infilename, buf)
                 buf.seek(0)
                 self._open_app(buf, is_filelike=True)
                 common.app_tree.app.saved = False
             except Exception, msg:
-                import traceback; traceback.print_exc()
-                wx.MessageBox(_("An exception occurred while importing file "
-                                "\"%s\".\nThis is the error message associated "
-                                "with it:\n        %s\n"
-                                "For more details, look at the full traceback "
-                                "on the console.\nIf you think this is a "
-                                "wxGlade bug, please report it.") % \
-                              (infile, msg), _("Error"),
-                              wx.OK|wx.CENTRE|wx.ICON_ERROR)
+                common.message.exception(
+                    _('An exception occurred while importing file "%s".') % \
+                        infilename.encode('ascii', 'replace')
+                    )                
+                wx.MessageBox(
+                    _("An exception occurred while importing file "
+                      "\"%s\".\nThis is the error message associated "
+                      "with it:\n        %s\n"
+                      "For more details, look at the full traceback "
+                      "on the console.\nIf you think this is a "
+                      "wxGlade bug, please report it.") % \
+                      (infilename, msg),
+                    _("Error"),
+                    wx.OK|wx.CENTRE|wx.ICON_ERROR
+                    )
 
     def manage_templates(self, event):
         to_edit = template.manage_templates()
