@@ -1,7 +1,7 @@
 # frame.py: wxFrame and wxStatusBar objects
-# $Id: frame.py,v 1.45 2007/03/27 07:02:00 agriggio Exp $
 #
 # Copyright (c) 2002-2007 Alberto Griggio <agriggio@users.sourceforge.net>
+#
 # License: MIT (see license.txt)
 # THIS PROGRAM COMES WITH NO WARRANTY
 
@@ -22,7 +22,7 @@ class EditStatusBar(EditBase):
                           custom_class=False, show=False)
         # style property
         self.style_pos  = (wx.ST_SIZEGRIP,)
-        style_labels = ('#section#Style', 'wxST_SIZEGRIP')
+        style_labels = ('#section#' + _('Style'), 'wxST_SIZEGRIP')
         self.access_functions['style'] = (self.get_style, self.set_style)
         self.properties['style'] = CheckListProperty(self, 'style', None,
                                                      style_labels)
@@ -75,7 +75,7 @@ class EditStatusBar(EditBase):
         sizer.Fit(page)
         page.SetSize(self.notebook.GetClientSize())
         sizer.Layout()
-        self.notebook.AddPage(page, "Common")
+        self.notebook.AddPage(page, _("Common"))
         self.property_window.Layout()
         prop.set_col_sizes([190, 0])
 
@@ -106,11 +106,10 @@ class EditStatusBar(EditBase):
             if self.widget: self.widget.Hide()
             EditBase.remove(self)
         else:
-            if misc.check_wx_version(2, 6):
-                if EditStatusBar._hidden_frame is None:
-                    EditStatusBar._hidden_frame = wx.Frame(None, -1, "")
-                if self.widget is not None:
-                    self.widget.Reparent(EditStatusBar._hidden_frame)
+            if EditStatusBar._hidden_frame is None:
+                EditStatusBar._hidden_frame = wx.Frame(None, -1, "")
+            if self.widget is not None:
+                self.widget.Reparent(EditStatusBar._hidden_frame)
             self.widget = None
 
     def popup_menu(self, *args):
@@ -186,7 +185,7 @@ class EditFrame(TopLevelBase):
 
         self.access_functions['icon'] = (self.get_icon, self.set_icon)
         prop = self.properties
-        style_labels = ['#section#Style', 'wxDEFAULT_FRAME_STYLE',
+        style_labels = ['#section#' + _('Style'), 'wxDEFAULT_FRAME_STYLE',
                         'wxICONIZE', 'wxCAPTION',
                         'wxMINIMIZE', 'wxMINIMIZE_BOX', 'wxMAXIMIZE',
                         'wxMAXIMIZE_BOX', 'wxSTAY_ON_TOP', 'wxSYSTEM_MENU',
@@ -207,28 +206,34 @@ class EditFrame(TopLevelBase):
                           wx.NO_FULL_REPAINT_ON_RESIZE,
                           wx.FULL_REPAINT_ON_RESIZE,
                           wx.TAB_TRAVERSAL, wx.CLIP_CHILDREN]
-        if misc.check_wx_version(2, 5):
-            style_labels.insert(5, 'wxCLOSE_BOX')
-            self.style_pos.insert(4, wx.CLOSE_BOX)
+        style_labels.insert(5, 'wxCLOSE_BOX')
+        self.style_pos.insert(4, wx.CLOSE_BOX)
         prop['style'] = CheckListProperty(self, 'style', None, style_labels)
         # menubar property
         prop['menubar'] = CheckBoxProperty(self, 'menubar', None,
-                                           'Has MenuBar')
+                                           _('Has MenuBar'))
         # statusbar property
         prop['statusbar'] = CheckBoxProperty(self, 'statusbar', None,
-                                             'Has StatusBar')
+                                             _('Has StatusBar'))
         # toolbar property
         prop['toolbar'] = CheckBoxProperty(self, 'toolbar', None,
-                                           'Has ToolBar')
+                                           _('Has ToolBar'))
         # icon property
         prop['icon'] = FileDialogProperty(self, 'icon', None,
                                           style=wx.OPEN|wx.FILE_MUST_EXIST,
-                                          can_disable=True)
+                                          can_disable=True, label=_("icon"))
         # centered property
         self.centered = False
         self.access_functions['centered'] = (self.get_centered,
                                              self.set_centered)
-        prop['centered'] = CheckBoxProperty(self, 'centered', None)
+        prop['centered'] = CheckBoxProperty(self, 'centered', None,
+                                            label=_("centered"))
+        # size hints property
+        self.sizehints = False
+        self.access_functions['sizehints'] = (self.get_sizehints,
+                                              self.set_sizehints)
+        prop['sizehints'] = CheckBoxProperty(self, 'sizehints', None,
+                                             label=_('Set Size Hints'))
 
     def create_widget(self):
         if self.parent: w = self.parent.widget
@@ -258,6 +263,7 @@ class EditFrame(TopLevelBase):
         prop['title'].display(panel)
         prop['icon'].display(panel)
         prop['centered'].display(panel)
+        prop['sizehints'].display(panel)
         prop['menubar'].display(panel)
         prop['toolbar'].display(panel)
         try:
@@ -271,6 +277,7 @@ class EditFrame(TopLevelBase):
         szr.Add(prop['title'].panel, 0, wx.EXPAND)
         szr.Add(prop['icon'].panel, 0, wx.EXPAND)
         szr.Add(prop['centered'].panel, 0, wx.EXPAND)
+        szr.Add(prop['sizehints'].panel, 0, wx.EXPAND)
         szr.Add(prop['menubar'].panel, 0, wx.EXPAND)
         szr.Add(prop['toolbar'].panel, 0, wx.EXPAND)
         if sbprop:
@@ -376,7 +383,8 @@ class EditFrame(TopLevelBase):
             if self.icon and not (self.icon.startswith('var:') or
                                   self.icon.startswith('code:')):
                 # setting icon
-                bmp = wx.Bitmap(self.icon, wx.BITMAP_TYPE_ANY)
+                icon = misc.get_relative_path(self.icon)
+                bmp = wx.Bitmap(icon, wx.BITMAP_TYPE_ANY)
                 if not bmp.Ok():
                     self.set_icon("")
                 else:
@@ -385,7 +393,11 @@ class EditFrame(TopLevelBase):
                     self.widget.SetIcon(icon) 
             else:
                 # removing icon
-                self.widget.SetIcon(wx.NullIcon)
+                icon = wx.EmptyIcon()
+                import os
+                xpm = os.path.join(common.icons_path, 'frame.xpm')
+                icon.CopyFromBitmap(misc.get_xpm_bitmap(xpm))
+                self.widget.SetIcon(icon)
 
     def get_centered(self):
         return self.centered
@@ -394,6 +406,13 @@ class EditFrame(TopLevelBase):
         try: self.centered = bool(int(value))
         except ValueError: pass
 
+    def get_sizehints(self):
+        return self.sizehints
+
+    def set_sizehints(self, value):
+        try: self.sizehints = bool(int(value))
+        except ValueError: pass
+
 # end of class EditFrame
 
 
@@ -415,7 +434,7 @@ def builder(parent, sizer, pos, number=[0]):
     """
     class Dialog(wx.Dialog):
         def __init__(self):
-            wx.Dialog.__init__(self, None, -1, 'Select frame class')
+            wx.Dialog.__init__(self, None, -1, _('Select frame class'))
             if common.app_tree.app.get_language().lower() == 'xrc':
                 self.klass = 'wxFrame'
             else:
@@ -424,14 +443,14 @@ def builder(parent, sizer, pos, number=[0]):
                 number[0] += 1
             self.base = 0
             base_prop = RadioProperty(self, 'base class', self,
-                                      ['wxFrame', 'wxMDIChildFrame'])
-            klass_prop = TextProperty(self, 'class', self)
+                                      ['wxFrame', 'wxMDIChildFrame'], label=_("base class"))
+            klass_prop = TextProperty(self, 'class', self, label=_("class"))
             szr = wx.BoxSizer(wx.VERTICAL)
             szr.Add(base_prop.panel, 0, wx.ALL|wx.EXPAND, 5)
             szr.Add(klass_prop.panel, 0, wx.EXPAND)
             btnbox = wx.BoxSizer(wx.HORIZONTAL)
-            btnOK = wx.Button(self, wx.ID_OK, 'OK')
-            btnCANCEL = wx.Button(self, wx.ID_CANCEL, 'Cancel')
+            btnOK = wx.Button(self, wx.ID_OK, _('OK'))
+            btnCANCEL = wx.Button(self, wx.ID_CANCEL, _('Cancel'))
             btnbox.Add(btnOK, 0, wx.ALL, 3)
             btnbox.Add(btnCANCEL, 0, wx.ALL, 3)
             btnOK.SetFocus()
@@ -439,6 +458,7 @@ def builder(parent, sizer, pos, number=[0]):
             self.SetAutoLayout(True)
             self.SetSizer(szr)
             szr.Fit(self)
+            self.CenterOnScreen()
             
         def undo(self):
             if number[0] > 0:
@@ -490,7 +510,7 @@ def _make_builder(base_class):
     def xml_builder(attrs, parent, sizer, sizeritem, pos=None):
         from xml_parse import XmlParsingError
         try: label = attrs['name']
-        except KeyError: raise XmlParsingError, "'name' attribute missing"
+        except KeyError: raise XmlParsingError, _("'name' attribute missing")
         frame = base_class(label, parent, wx.NewId(), "",
                            common.property_panel,
                            show=False)
@@ -506,7 +526,7 @@ def _make_builder(base_class):
 ##     """
 ##     from xml_parse import XmlParsingError
 ##     try: label = attrs['name']
-##     except KeyError: raise XmlParsingError, "'name' attribute missing"
+##     except KeyError: raise XmlParsingError, _("'name' attribute missing")
 ##     frame = EditFrame(label, parent, wx.NewId(), label, common.property_panel,
 ##                       show=False)
 ##     node = Tree.Node(frame)
@@ -541,9 +561,12 @@ def initialize():
     # add statusbar icon to WidgetTree
     from tree import WidgetTree
     import os.path
-    WidgetTree.images['EditStatusBar'] = os.path.join(common.wxglade_path,
-                                                      'icons/statusbar.xpm')
-    WidgetTree.images['EditMDIChildFrame'] = os.path.join(common.wxglade_path,
-                                                          'icons/frame.xpm')
-       
+    WidgetTree.images['EditStatusBar'] = os.path.join(
+        common.icons_path,
+        'statusbar.xpm'
+        )
+    WidgetTree.images['EditMDIChildFrame'] = os.path.join(
+        common.icons_path,
+        'frame.xpm'
+        )       
     return common.make_object_button('EditFrame', 'icons/frame.xpm', 1)
