@@ -1,10 +1,10 @@
 #!/usr/bin/env python
-# wxglade.py: entry point of wxGlade
-#
-# Copyright (c) 2002-2007 Alberto Griggio <agriggio@users.sourceforge.net>
-#
-# License: MIT (see license.txt)
-# THIS PROGRAM COMES WITH NO WARRANTY
+"""
+Entry point of wxGlade
+
+@copyright: 2002-2007 Alberto Griggio <agriggio@users.sourceforge.net>
+@license: MIT (see license.txt) - THIS PROGRAM COMES WITH NO WARRANTY
+"""
 
 import os
 import sys
@@ -12,10 +12,11 @@ import gettext
 import common
 import optparse
 
-
 t = gettext.translation(domain="wxglade", localedir="locale", fallback=True)
 t.install("wxglade")
 
+import errors
+
 def _fix_path(path):
     """\
     Returns an absolute version of path, accroding to the invoking dir of
@@ -27,6 +28,11 @@ def _fix_path(path):
         #getenv('WXGLADE_INVOKING_DIR', '.'), path)
     return path
 
+def error(msg):
+    """\
+    Print an error message at stderr
+    """
+    print >> sys.stderr, _("ERROR: %s") % msg
 
 def parse_command_line():
     """\
@@ -106,11 +112,11 @@ wxGlade home page: <http://wxglade.sourceforge.net/>""")
         if len(args) == 1:
             options.start_gui = False
         elif len(args) == 0:
-            print >> sys.stderr, _("ERROR: No wxg file given!\n")
+            error(_("No wxg file given!\n"))
             parser.print_help()
             sys.exit(1)
         else:
-            print >> sys.stderr, _("ERROR: Too many wxg files given!\n")
+            error(_("Too many wxg files given!\n"))
             parser.print_help()
             sys.exit(1)
     else:
@@ -130,26 +136,30 @@ def command_line_code_generation(filename, language, out_path=None):
     @param out_path: output file / output directory
     @type out_path:  String
     """
-    common.use_gui = False # don't import wxPython.wx
-    # use_gui has to be set before importing config
-    import config
-    config.init_preferences()
-    common.load_code_writers()
-    common.load_widgets()
-    common.load_sizers()
-
     from xml_parse import CodeWriter
     if not common.code_writers.has_key(language):
-        print >> sys.stderr, \
-            _('Error: no writer for language "%s" available') % language
-        sys.exit(1)
+        error(_('No writer for language "%s" available') % language)
 
     writer = common.code_writers[language]
-    CodeWriter(
-        writer=writer,
-        input=filename,
-        out_path=out_path,
-        )
+    try:
+        CodeWriter(
+            writer=writer,
+            input=filename,
+            out_path=out_path,
+            )
+    except (errors.WxgOutputDirectoryNotExist,
+            errors.WxgOutputDirectoryNotWritable,
+            errors.WxgOutputPathIsDirectory,
+            ), inst:
+        error(inst)
+        sys.exit(1)
+    except Exception:
+        common.message.exception(_('Internal Error'))
+        error(
+            _("An exception occurred while generating the code for the application.\n"
+              "If you think this is a wxGlade bug, please report it.")
+             )
+        sys.exit(1)
     sys.exit(0)
 
 
@@ -168,20 +178,13 @@ def determine_wxglade_path():
     return os.path.dirname(os.path.abspath(root))
 
 
-def run_main():
+def init_stage1():
     """\
-    This main procedure is started by calling either wxglade.py or
-    wxglade.pyw on windows
+    Initialise paths for wxGlade (first stage)
+
+    Path initialisation is splitted because the test suite doesn't work
+    with proper initialised paths.
     """
-    # check command line parameters first
-    options = parse_command_line()
-
-    # print versions 
-    print _("Starting wxGlade version %s on Python %s") % (
-        common.version,
-        common.py_version,
-        )
-
     # prepend the widgets dir to the
     wxglade_path = determine_wxglade_path()
     
@@ -195,27 +198,29 @@ def run_main():
     common.templates_path = os.path.join(wxglade_path, 'templates')
     common.tutorial_file  = os.path.join(common.docs_path, 'html', 'index.html')
 
-    # search credits file at two different locations
-    # - <wxglade_path>/docs/credits.txt for linux packages
-    # - <wxglade_path>/credits.txt at Windows or started from source directory
-    if os.path.exists(os.path.join(common.wxglade_path, 'credits.txt')):
-        common.credits_file = os.path.join(common.wxglade_path, 'credits.txt')
-    elif os.path.exists(os.path.join(common.docs_path, 'credits.txt')):
-        common.credits_file = os.path.join(common.docs_path, 'credits.txt')
-    else:
-        print _('ERROR: credits file "credits.txt" not found!')
-        common.credits_file = None
-
-    # search license file at two different locations
-    # - <wxglade_path>/docs/license.txt for linux packages
-    # - <wxglade_path>/license.txt at Windows or started from source directory
-    if os.path.exists(os.path.join(common.wxglade_path, 'license.txt')):
-        common.license_file = os.path.join(common.wxglade_path, 'license.txt')
-    elif os.path.exists(os.path.join(common.docs_path, 'license.txt')):
-        common.license_file = os.path.join(common.docs_path, 'license.txt')
-    else:
-        print _('ERROR: license file "license.txt" not found!')
-        common.license_file = None
+    # search files credits.txt and license.txt at different locations
+    # - <wxglade_path>/docs   for linux packages
+    # - <wxglade_path>   at Windows or started from source directory
+    # - <wxglade_path>/./../../../share/doc/wxglade/   for local installations
+    # BTW: <wxglade_path> is something like /.../lib/python2.7/site-packages/wxglade
+    common.credits_file = None
+    common.license_file = None
+    for searchdir in [
+        common.wxglade_path,
+        common.docs_path,
+        os.path.join(common.wxglade_path, '../../../../share/doc/wxglade'),
+        ]:
+        searchdir = os.path.normpath(searchdir) 
+        credits_file = os.path.join(searchdir, 'credits.txt')
+        license_file = os.path.join(searchdir, 'license.txt')
+        if os.path.exists(credits_file):
+            common.credits_file = credits_file
+        if os.path.exists(license_file):
+            common.license_file = license_file
+    if not common.credits_file:
+        error(_('Credits file "credits.txt" not found!'))
+    if not common.license_file:
+        error(_('License file "license.txt" not found!'))
 
     # print used paths
     print _('Base directory:             %s') % common.wxglade_path
@@ -230,7 +235,60 @@ def run_main():
     # adapt application search path
     sys.path = [common.wxglade_path, common.widgets_path] + sys.path
 
+
+def init_stage2(use_gui):
+    """\
+    Initialise the remaining (non-path) parts of wxGlade (second stage)
+
+    @param use_gui: Starting wxGlade GUI
+    @type use_gui:  Boolean
+    """
+    common.use_gui = use_gui
+    if use_gui:
+        # ensure minimal wx version
+        if not hasattr(sys, 'frozen') and \
+           'wxversion' not in sys.modules and \
+           'wx' not in sys.modules:
+            import wxversion
+            wxversion.ensureMinimal("2.6")
+        
+        # store current platform (None is default)
+        import wx
+        common.platform = wx.Platform
+
+        # codewrites, widgets and sizers are loaded in class main.wxGladeFrame
+    else:
+        # use_gui has to be set before importing config
+        import config
+        config.init_preferences()
+        common.load_code_writers()
+        common.load_widgets()
+        common.load_sizers()
+
+
+def run_main():
+    """\
+    This main procedure is started by calling either wxglade.py or
+    wxglade.pyw on windows
+    """
+    # check command line parameters first
+    options = parse_command_line()
+
+    # print versions 
+    print _("Starting wxGlade version %s on Python %s") % (
+        common.version,
+        common.py_version,
+        )
+
+    # initialise wxGlade (first stage)
+    init_stage1()
+
+    # initialise wxGlade (second stage)
+    init_stage2(options.start_gui)
+
     if options.start_gui:
+        # late import of main (imported wx) for using wxversion  in
+        # init_stage2()
         import main
         main.main(options.filename)
     else:
